import { Component, Input, Output, EventEmitter, Inject } from '@angular/core';
import { FormControl } from '@angular/forms'
import { Observable } from 'rxjs';
import { map, startWith, debounceTime } from 'rxjs/operators';
import { MatDialogRef, MAT_DIALOG_DATA, MatDialog } from '@angular/material/dialog';


/**
 * How to use AutoComplete
 * 1.create new tag name
 *    <auto-complete></auto-complete>
 * 2.add properties
 *    - config [Required]
 *    - data [Required]
 *    - textInput [Not-required][Input]
 *    - disabled [Not-required][Input]
 *    - tabindex [Not-required][Input]
 *    - datacallback [Required]
 * 
 * ***config property's model
 *  1. ColName: Key of object to show in screen.
 *  2. ColValue: Value for returning to callback (if this is not defined, return value will be a selected object.)
 *  3. PlaceholderText: A name that user understand what to do with this auto-complete
 *  4. MinLengthSearch
 */


@Component({
  selector: 'auto-complete',
  template: '<button type="button" mat-icon-button (click)="OpenDialog()" [disabled]="disabled" tabindex="{{tabindex}}"><i class="material-icons">arrow_drop_down</i></button>',
  styleUrls: ['./auto-complete-btn.component.scss']
})

export class AutoCompleteComponent {

  constructor(public dialog: MatDialog) { }

  @Input() config: any;
  @Input() data: any;
  
  private _textInput: string;
  @Input()
  get textInput(): string {return this._textInput};
  set textInput(textInput: string) {
    this._textInput = textInput?.toString().trim() || "";
  }

  @Output() datacallback = new EventEmitter<string>();
  @Output() cancelcallback = new EventEmitter<string>();
  @Input() disabled: boolean = false;
  @Input() tabindex: number = -1;

  private dialogRef: any;

  ngOnDestroy() {
    setTimeout(() => {
      this.dialogRef ? this.dialogRef.close() : null;
    }, 100)
  }

  OpenDialog() {
    this.dialogRef = this.dialog.open(AutoCompleteDialog, {
      width: '40%',
      data: {
        config: this.config,
        data: this.data,
        datacallback: this.datacallback,
        cancelcallback: this.cancelcallback
      }
    })

    this.dialogRef.afterClosed().subscribe(result => { })
  }

  query() {
    if (!this.disabled) {
      let matchedValue: any;
      let matchedResult: any;

      if (this.config.ColValue) {
        matchedValue = this.data.find(v => v[this.config.ColValue] === (this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput));
      } else {
        matchedValue = this.data.find(v => {
          let mIndex = Object.keys(v).findIndex(k => {
            return v[k] == (this.config.UpperCase ? this.textInput.toString().toUpperCase() : this.textInput);
          })
          return mIndex !== -1;
        })
      }

      if (matchedValue) {
        matchedResult = (this.config.ColValue) ? matchedValue[this.config.ColValue] : matchedValue;
      } else {
        matchedResult = (this.config.ColValue) ? null : {};
      }

      this.datacallback.emit(matchedResult);
    }
  }


}

@Component({
  selector: 'auto-complete-dialog',
  templateUrl: './auto-complete.component.html',
  styleUrls: ['./auto-complete.component.scss']
})
export class AutoCompleteDialog {

  public states: any;
  public setting: any;
  public selectedValue: any;
  public defaultMinLengthSearch: number = 3;

  myControl = new FormControl();
  filteredOptions: Observable<any>;

  constructor(
    public dialogRef: MatDialogRef<AutoCompleteDialog>,
    @Inject(MAT_DIALOG_DATA) public dialogData: any
  ) { }

  ngOnInit() {
    this.setting = this.dialogData.config;
    this.states = this.dialogData.data
    this.filteredOptions = this.myControl.valueChanges
      .pipe(
        startWith(''),
        debounceTime(500),
        //map(value => value.length >= (this.setting.MinLengthSearch || this.defaultMinLengthSearch) && this._filter(value))
        map(value => this._filter(value))
      );
  }

  _filter(value: string): string[] {

    if (value.length < this.setting.MinLengthSearch) return;

    const filterValue = value.toLowerCase();
    let prop = this.setting.ColName;
    return this.states.filter(option => option[prop].toLowerCase().includes(filterValue));
  }

  ClickCallback() {
    this.dialogData.datacallback.emit(this.selectedValue);
    this.dialogRef.close();
  }

  OnCancelClick() {
    if(this.dialogData.cancelcallback) {
      this.dialogData.cancelcallback.emit();
    }
    this.dialogRef.close();
  }

  SetValue(evt: any) {
    this.myControl.setValue(evt.option.value[this.setting.ColName]);
    this.selectedValue = (this.setting.ColValue) ? evt.option.value[this.setting.ColValue] : evt.option.value;
  }

  SetClearValue() {
    this.selectedValue = (this.setting.ColValue) ? null : {};
    this.myControl.setValue('')
  }
}
